<?php

function debug_to_console($output, $with_script_tags = true) {
    $js_code = 'console.log(' . json_encode('Debug: '.$output, JSON_HEX_TAG) . ');';
    if ($with_script_tags) {
        $js_code = '<script>' . $js_code . '</script>';
    }
    echo $js_code;
}

include_once dirname(__FILE__) . '/lib/BesteronClass.php';

class WC_Besteron_Payment_Gateway extends WC_Payment_Gateway {

    // variable for support banks
    public $support_banks = array();
    // variable for settings
    public $besteron_demo_var;
    public $debug_mode;
    public $multi_curr_var;
    public $enabled_card_var = "";
    public $enabled_bank_buttons_var = "";
    public $payment_type = Besteron::PAYMENT_TYPE_DEFAULT;
    public $support_sk = "yes";
    public $support_cz = "no";
    public $demo = "no";
    public $apple_pay;
    public $multi_curr = "yes";
    public $enabled_card = "yes";
    public $enabled_bank_buttons = "yes";
    public $debug = "no";
    public $validity_time = 3600;
    public $additional_param_name = "";
    public $additional_param_value = "";

    // Setup our Gateway's id, description and other values
    function __construct() {

        // The global ID for this Payment method
        $this->id = "besteron";

        // The Title shown on the top of the Paymegnt Gateways Page next to all the other Payment Gateways
        $this->method_title = __("Redirect Online Payment", 'besteron-payment-gateway');

        // The description for this Payment Gateway, shown on the actual Payment options page on the backend
        $this->method_description = __("Redirect Online Payment Gateway Plug-in for WooCommerce", 'besteron-payment-gateway');

        // The title to be used for the vertical tabs that can be ordered top to bottom
        $this->title = __('Online Payment Gateway', 'besteron-payment-gateway');

        // If you want to show an image next to the gateway's name on the frontend, enter a URL to an image.
        //$this->icon = null; // without logo
        $this->icon = plugins_url(basename(plugin_dir_path(__FILE__)), basename(__FILE__)).'/assets/images/besteron/besteron_logo_small.png';

        // Bool. Can be set to true if you want payment fields to show on the checkout$order->status
        // if doing a direct integration, which we are doing in this case
        $this->has_fields = true;

        // Supports the default credit card form
        $this->supports = array();

        // After init_settings() is called, you can get the settings and load them into variables, e.g:
        // $this->title = $this->get_option( 'title' );
        $this->init_settings();

        // Turn these settings into variables we can use
        foreach ($this->settings as $setting_key => $value) {
            $this->$setting_key = $value;
        }

        // add scripts
        if ( $this->payment_type == 'inline' ) {
            add_action( 'wp_enqueue_scripts', array( $this, 'scripts' ) );
        }

        $this->payment_type = 'redirect';

        // add admin scripts
        add_action( 'admin_enqueue_scripts', function() { 
            wp_enqueue_script( 'wc_besteron_admin', plugins_url(basename(plugin_dir_path(__FILE__)). '/assets/js/besteron_functions.js?v=2' ), array( 'jquery' ) ); 
        } );

        // SET variable
        // support banks
        if ( $this->support_sk == "yes" ) { array_push($this->support_banks, 'SK'); }
        if ( $this->support_cz == "yes" ) { array_push($this->support_banks, 'CZ'); }
        // is demo ?
        $this->besteron_demo_var = ( $this->demo == "yes" ) ? true : false;
        // is multi currency ?
        $this->multi_curr_var = ( $this->multi_curr == "yes" ) ? true : false;
        $replacedurl = site_url();
        $replacedurl = str_replace('https://', '', $replacedurl);
        $replacedurl = str_replace('.', '', $replacedurl);
        $key_enabled = $replacedurl . '_enabled_card';

        // is card enable ?
        $this->enabled_card_var = ( $this->$key_enabled == "yes" ) ? true : false;
        // is bank buttons enable ?

        $replacedurl = site_url();
        $replacedurl = str_replace('https://', '', $replacedurl);
        $replacedurl = str_replace('.', '', $replacedurl);
        $key_buttons = $replacedurl . '_enabled_bank_buttons';

        $this->enabled_bank_buttons_var = ( $this->$key_buttons == "yes" ) ? true : false;
        // is plugin in debug mode ?
        $this->debug_mode = ( $this->debug == "yes" ) ? true : false;

      //  $this->apple_pay = $this->

        // This basically defines your settings which are then loaded with init_settings()
        $this->init_form_fields();

        // Lets check for SSL
        //add_action('admin_notices', array($this, 'do_ssl_check'));
        // Save settings
        if (is_admin()) {
            // Versions over 2.0
            // Save our administration options. Since we are not going to be doing anything special
            // we have not defined 'process_admin_options' in this class so the method in the parent
            // class will be used instead
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        }

        //TODO: see it
        add_action('woocommerce_thankyou_' . $this->id, array(&$this, 'thankyou_page'));
        //add_action('woocommerce_receipt_'.$this->id, array(&$this, 'receipt_page'));
    }


    // load custom scripts
    function scripts(){
        // payment gateway scripts
        wp_enqueue_style('wc_besteron', plugins_url(basename(plugin_dir_path(__FILE__)).'/assets/css/besteron_payment.css?v=2'));

        // debug info to console
        if ( $this->debug_mode ) debug_to_console('Demo mod: '.$this->demo); 

        wp_enqueue_script( 'wc_besteron' );
    }

    // return CID or KEY by bank BIC and currency
    // for support mulicountry banks
    public function getBankCIDKEY($type, $bic, $currency) {
        $country = '';

        if ( $this->payment_type == 'inline' ) {
            $country = $this->inline_payment_cid_key;
            if ( $country == 'auto') {
                foreach (Besteron::getSupportBanksPossible() as $key => $pos_bank) {
                    if ( $pos_bank['curr'] == $currency ) $country = $key;
                }
            }
        } else {
            if ($bic != 'SPOROCARD') {
                // if bank, get by currency
                foreach (Besteron::getSupportBanksPossible() as $key => $pos_bank) {
                    if ( $pos_bank['curr'] == $currency ) $country = $key;
                }
            } else {
                // for credit card - get first enable support bank
                $country = $this->support_banks[0];
            }
        }
        $country = strtolower($country);

        if($currency == 203){
            $country = 'cz';
        }

         $replacedurl = site_url();
            $replacedurl = str_replace('https://', '', $replacedurl);
            $replacedurl = str_replace('.', '', $replacedurl);

        if ( ($country != 'auto') && ($country != '') && ($country != Besteron::LANG_DEFAULT) ) {
            return trim($this->{$replacedurl.'besteron_'.$type.'_'.$country});
        } else {
            return trim($this->{$replacedurl.'besteron_'.$type});
        }
    }


    // language for Besteron gateway
    // @return string
    private function getSiteLanguage() {
        // get Besteron language from settings
        $lang = $this->gateway_language;
        // auto language
        if ( 'wp_auto' === $lang ) {
            // get language from WP locale
            //$lang = substr(get_bloginfo('language'), 0, 2); // alternative
            list( $lang ) = explode( '_', get_locale(), 2 );
            if ( 'cs' === $lang ) {
                // repair CZ language
                $lang = 'cs';
            }
        }
        return $lang;
    }

    // Thank you Page
    function thankyou_page($order_id) {
        global $woocommerce;

        // debug info to console
        if ( $this->debug_mode ) debug_to_console('function thankyou_page start'); 

        // debug_to_console($_GET['transactionId']);

        $order = new WC_Order($order_id);

        if($order->get_status() == 'completed'){
            $woocommerce->cart->empty_cart();
            $redirect_url = $this->get_return_url($order);
            
            return;
        }

        if($order->get_status() == 'processing'){
            $woocommerce->cart->empty_cart();
            $redirect_url = $this->get_return_url($order);
            
            return;
        }

        $wc_currency = $order->get_currency();

        $order_currency = array_search($wc_currency, Besteron::getCurrList()); 

        if(isset($_GET['transactionId'])){

            $transactionId = $_GET['transactionId'];
            update_post_meta($order_id,'transaction_id',$transactionId );

            $replacedurl = site_url();
            $replacedurl = str_replace('https://', '', $replacedurl);
            $replacedurl = str_replace('.', '', $replacedurl);
            $key = $replacedurl . '_inline_payment_type';

            $cid = $this->getBankCIDKEY('cid', '', $order_currency); // get CID
            $key = $this->getBankCIDKEY('api', '', $order_currency); // get KEY
            $lang_best = $this->getSiteLanguage();
            $ru = $this->get_return_url($order);
            $payment_type = $this->payment_type;
            $currency = array_search($wc_currency, Besteron::getCurrList()); 
            $amnt = $order->get_total();
            $inline_payment_type = $this->$key; // CARD or CARDANDBUTTON

            $besteron = new BesteronRequest(
                array(
                    Besteron::PARAM_CID     => $cid,
                    Besteron::PARAM_KEY     => $key,
                    Besteron::PARAM_DEMO_MODE => $this->besteron_demo_var,
                    Besteron::PARAM_AMNT    => floatval($amnt),
                    Besteron::PARAM_CURR    => $currency,
                    Besteron::PARAM_VS      => strval($order->id),
                    Besteron::PARAM_RU      => $ru,
                    Besteron::PARAM_EMAIL   => $order->get_billing_email(),
                    Besteron::PARAM_LANG    => $lang_best,
                    Besteron::PARAM_PAYMENT_TYPE => $payment_type,
                    Besteron::PARAM_INLINE_PAYMENT_TYPE => $inline_payment_type,
                    Besteron::VALIDITY_TIME => $this->validity_time,
                    Besteron::ADDITIONAL_PARAM_NAME => $this->additional_param_name,
                    Besteron::ADDITIONAL_PARAM_VALUE => $this->additional_param_value
                )
            );

            $status = $besteron->getTransactionStatus($transactionId);

            // if success
           
            if($status == 'Completed'){
                if($order->get_status()!='processing'){

                    $order->add_order_note(__('Besteron has changed the payment status to Completed.', 'besteron-payment-gateway'));
                }
                $order->update_status('processing');
                $order->payment_complete();    
              //  $order->save();       
                $woocommerce->cart->empty_cart();
                $redirect_url = $this->get_return_url($order);
                
            }
            else if($status == 'Created'){
                if($order->get_status()!='pending'){
                    $order->add_order_note(__('Besteron has changed the payment status to Created.', 'besteron-payment-gateway'));
                }
                $order->update_status('pending');
            //  $order->save();
                $redirect_url = $this->get_return_url($order);
                
            }
            else if($status == 'ManualAttentionRequired'){
                if($order->get_status()!='pending'){
                    $order->update_status('pending');
                    $order->add_order_note(__('Besteron has changed the payment status to Manual Attention Required.', 'besteron-payment-gateway'));
                }
            //  $order->save();
                $redirect_url = $this->get_return_url($order);
                
            }
            else if($status == 'Invalid'){
                if($order->get_status()!='failed'){
                    $order->add_order_note(__('Besteron has changed the payment status to Invalid.', 'besteron-payment-gateway'));
                }
                $order->update_status('failed');
            //  $order->save();
                $redirect_url = $this->get_return_url($order);
                
            }
            else if($status == 'Error'){
                if($order->get_status()!='failed'){
                    $order->add_order_note(__('Besteron has changed the payment status to Error.', 'besteron-payment-gateway'));
                }
                $order->update_status('failed');
                $redirect_url = $this->get_return_url($order);
                
            }
             else if($status == 'Canceled'){
                if($order->get_status()!='pending'){
                    $order->update_status('failed');
                }
                $order->add_order_note(__('Besteron has changed the payment status to Canceled.', 'besteron-payment-gateway'));
             //   $order->save();
                $redirect_url = $this->get_return_url($order);
                
            }
            else if($status == 'Timeouted'){
                if($order->get_status()!='pending'){
                    $order->add_order_note(__('Besteron has changed the payment status to Timeouted.', 'besteron-payment-gateway'));
                }
                $order->update_status('failed');
            //  $order->save();
                $redirect_url = $this->get_return_url($order);
                
            }
           
            else if($status == 'WaitingForConfirmation'){
                if($order->get_status()!='pending'){
                    $order->add_order_note(__('Besteron has changed the payment status to Waiting for confirmation.', 'besteron-payment-gateway'));
                }
                $order->update_status('pending');
                $redirect_url = $this->get_return_url($order);
                
            }

        }
        
        //check is result
        $result = array_key_exists(Besteron::PARAM_RESULT, $_REQUEST) ? $_REQUEST[Besteron::PARAM_RESULT] : null;
        if (!$result) {
            return;
        }
        // debug info to console
        if ( $this->debug_mode ) {
            debug_to_console('check is result');
            debug_to_console( $result );
        }
        
        // get bic from param_type
        $bic = $_REQUEST[Besteron::PARAM_TYPE];

        $transactionId = null;
       
        //vlidate sign

        //ver 1.1.1 - nastavenie ID objednavky alebo Cislo objednavky - ak nieco meni cislo objednavky
        if ( $this->vs_type == "yes" ) {
            //ak je povolene cislo objednavky v settings besteron
            $vs = $order->get_order_number();
            if (!is_numeric($vs) || (strlen($vs) > 10)) {
                // ak variabilny symbol nie je cislo alebo je dlhsi ako 10znakov, tak to bude ID objednavky
                $vs = str_pad($order_id, 10, "0", STR_PAD_LEFT);
            }
        } else {
            $vs = str_pad($order_id, 10, "0", STR_PAD_LEFT);
        }

        $error = false;
        if (!$error && $vs != $_REQUEST[Besteron::PARAM_VS]) {
            $error = __('Incorrect vs ', 'besteron-payment-gateway'); // "nesuhlasi vs "
        }
        if (!$error && $order->get_total() != $_REQUEST[Besteron::PARAM_AMNT]) {
            $error = __('Incorrect amnt', 'besteron-payment-gateway'); // "nesuhlasi amnt"
        }


        if (!$error && $order_currency != $_REQUEST[Besteron::PARAM_CURR]) {
            $error = __('Incorrect curr', 'besteron-payment-gateway');
        }
        
        $cid = $this->getBankCIDKEY('cid', $bic, $order_currency); // get CID
        $key = $this->getBankCIDKEY('key', $bic, $order_currency); // get KEY

        if (!$error) {
            if ($result == Besteron::RESULT_OK || $result == Besteron::RESULT_FAIL) {
                //validate sign
                if ( $this->debug_mode ) debug_to_console('validate besteron sign, result: '.$result); //debug
                try {
                    $besteron = new BesteronResponse(
                        array(
                            Besteron::PARAM_CID     => $cid,
                            Besteron::PARAM_KEY     => $key,
                            Besteron::PARAM_DEMO_MODE => $this->besteron_demo_var,
                            Besteron::PARAM_AMNT   => $_REQUEST['AMNT'],
                            Besteron::PARAM_CURR   => $_REQUEST['CURR'],
                            Besteron::PARAM_VS     => $_REQUEST['VS'],
                            Besteron::PARAM_TYPE   => $_REQUEST['TYPE'],
                            Besteron::PARAM_RESULT => $result,
                            Besteron::PARAM_SIGN   => $_REQUEST['SIGN'],
                        )
                    );
                    if (!$besteron->isValid()) {
                        $error = 'invalid sign';
                        if ( $this->debug_mode ) debug_to_console('besteron '.$error); //debug
                    }
                } catch (Exception $e) {
                    $error = $e->getMessage();
                    if ( $this->debug_mode ) debug_to_console('invalid besteron sign, error: '.$error); //debug
                }
            } else {
                if ( $this->debug_mode ) debug_to_console('Error result thank_you: '.$result); //debug
                $error = __('Unknown server response:', 'besteron-payment-gateway') .' '. $result;
                $order->update_status('failed');
                $order->add_order_note(__('Payment transaction failed', 'besteron-payment-gateway').'. '.$error.' (type: '.$bic.')');
            }
        }

        if ($error) {
            $msg = __('An error occurred. The payment has not validated.', 'besteron-payment-gateway');
            if ( $this->debug_mode ) { //debug 
                debug_to_console('Error text after payment: '.$error); 
                $msg = $msg.' Error text: '.$error;
            }
            wc_add_notice($msg, 'error');
        }

        if ( ($result == 'OK') && (!$error) ) {
            $order->payment_complete();
            $woocommerce->cart->empty_cart();
            $redirect_url = $this->get_return_url($order);
            
        } else if ( ($result == Besteron::RESULT_FAIL) || ($result == 'FAIL') || ($error) ) { // TO DO: result is 'FAIL', not 'RESULT_FAIL' !!!
            $msg = __('Payment transaction failed', 'besteron-payment-gateway');
            if ( $this->debug_mode ) { //debug 
                debug_to_console('Error result after payment: '.$result); 
                $msg = $msg.' Error result: '.$result;
            }
            $order->update_status('failed');
            $order->add_order_note($msg);
            wc_add_notice($msg, 'error');
            $redirect_url = $this->get_return_url($order);
            
            exit;
        }
    }


    

    // Build the administration fields for this specific Gateway
    public function init_form_fields() {

        $this->form_fields = [
            'enabled' => array(
                'title' => __('Enable / Disable', 'besteron-payment-gateway'),
                'label' => __('Enable this payment gateway', 'besteron-payment-gateway'),
                'type' => 'checkbox',
                'default' => 'no',
            ),
            'title' => array(
                'title'       => __('Title', 'besteron-payment-gateway'),
                'type'        => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'besteron-payment-gateway'),
                'default'     => __('Online Payment', 'besteron-payment-gateway'),
                'desc_tip'    => true,
            ),
            'description' => array(
                'title'       => __('Description', 'besteron-payment-gateway'),
                'type'        => 'textarea',
                'description' => __('Pay with your credit card or banks buttons with Besteron payment gateway.', 'besteron-payment-gateway'),
                'default'     => __('Pay with your credit card or banks buttons with Besteron payment gateway.', 'besteron-payment-gateway'),
            ),
            'validity_time' => array(
                'title'       => __('Validity time', 'besteron-payment-gateway'),
                'type'        => 'number',
                'description' => __('Enter your validity time.', 'besteron-payment-gateway'),
                'default'     => '3600',
                'desc_tip' => __('The validity period of the payment request. It determines the time until the customer can make the payment.', 'besteron-payment-gateway')
            ), 
           'vs_type' => array(
                'title' => __('VS Type', 'besteron-payment-gateway'),
                'label' => __('Use Order Number instead of Order ID', 'besteron-payment-gateway'),
                'type' => 'checkbox',
                'description' => __('If you use a plugin that changes the Order Number. Important: order number must be a number, max. length 10!', 'besteron-payment-gateway'),
                'default' => '',
            ),
            'gateway_language' => array(
                'title' => __('Payment gateway language', 'besteron-payment-gateway'),
                'type' => 'select',
                'description' => __('Language in which show the payment gateway interface is displayed to the customer.', 'besteron-payment-gateway'),
                'desc_tip' => __('Select prefered language or select Use site language for get language from your website.', 'besteron-payment-gateway'),
                'default'       => Besteron::LANG_DEFAULT,
                'options'       => Besteron::getLangList()
            ),
            'allowed_curr' => array(
                'title'         => __( 'Allowed currencies', 'besteron-payment-gateway' ),
                'type'          => 'multiselect',
                'description'   => __( 'If you do not select any, will be used all supported.', 'besteron-payment-gateway' ),
                'desc_tip'   => __( 'Select all allowed currencies that can be used for payment.', 'besteron-payment-gateway' ),
                'class'         => 'wc-enhanced-select',
                'options'       => Besteron::getCurrList()
            ),        
            'debug' => array(
                'title' => __('Debug mode', 'besteron-payment-gateway'),
                'label' => __('Enable debug mode for diagnostic', 'besteron-payment-gateway'),
                'type' => 'checkbox',
                'default' => 'yes',
            ),
        
        ]; 

            $this->form_fields['support_sk'] = array(
                'title' => __('Support SK banks', 'besteron-payment-gateway'),
                'label' => __('Enable support for Slovak banks (Slovak CID/KEY)', 'besteron-payment-gateway'),
                'type' => 'checkbox',
                'default' => 'yes',
            );
            
            $replacedurl = site_url();
            $replacedurl = str_replace('https://', '', $replacedurl);
            $replacedurl = str_replace('.', '', $replacedurl);



            $this->form_fields[$replacedurl.'_inline_payment_type'] = array(
                'title'         => __( 'Inline Payment type', 'besteron-payment-gateway' ),
                'type'          => 'select',
                'description'   => __( 'Select payment option for inline gateway.', 'besteron-payment-gateway' ),
                'desc_tip'      => __( 'Select payment option that can be used for inline payment.', 'besteron-payment-gateway' ),
                'default'       => Besteron::INLINE_PAYMENT_TYPE_DEFAULT,
                'options'       => Besteron::getInlinePaymentTypeList()
            );               
            $this->form_fields[$replacedurl.'_enabled_card'] = array(
                'title'     => __('Allow credit card', 'besteron-payment-gateway'),
                'type'      => 'checkbox',
                'label'     => __('Enable credit card payment functionality', 'besteron-payment-gateway'),
                'desc_tip'  => __('If you want to use payments in credit cards.', 'besteron-payment-gateway'),
                'default'   => 'yes'
            );
            $this->form_fields[$replacedurl.'_enabled_bank_buttons'] = array(
                'title'     => __('Allow bank buttons', 'besteron-payment-gateway'),
                'type'      => 'checkbox',
                'label'     => __('Enable bank buttons payment functionality', 'besteron-payment-gateway'),
                'desc_tip'  => __('If you want to use bank buttons.', 'besteron-payment-gateway'),
                'default'   => 'yes'
            );
           
            $this->form_fields[$replacedurl.'_allowed_banks'] = array(
                'title'     => __( 'Allowed banks', 'besteron-payment-gateway' ),
                'type'      => 'multiselect',
                'description' => __( 'If you do not select any, will be used all banks supported.', 'besteron-payment-gateway' ),
                'desc_tip'  => __( 'Select all allowed banks that can be used for payment.', 'besteron-payment-gateway' ),
                'class'     => 'wc-enhanced-select',
                'options'   => Besteron::getBanks($this->support_banks)
            );  

            $this->form_fields[$replacedurl.'besteron_cid'] = array(
                'title' => __('CID (SK)', 'besteron-payment-gateway'),
                'type' => 'text',
                'desc_tip' => __('Your Merchant CID from your BESTERON account Integration page', 'besteron-payment-gateway'),
                'default' => '',
            );

            $this->form_fields[$replacedurl.'besteron_key'] = array(
                'title' => __('KEY (SK)', 'besteron-payment-gateway'),
                'type' => 'password',
                'desc_tip' => __('Your Merchant Key from your BESTERON account Integration page', 'besteron-payment-gateway'),
            );

            $this->form_fields[$replacedurl.'besteron_api'] = array(
                'title' => __('API (SK)', 'besteron-payment-gateway'),
                'type' => 'password',
                'desc_tip' => __('Enter the Besteron API key.', 'besteron-payment-gateway'),
            );


            $this->form_fields['support_cz'] = array(
                'title' => __('Support CZ banks', 'besteron-payment-gateway'),
                'label' => __('Enable support for Czech banks (Czech CID/KEY)', 'besteron-payment-gateway'),
                'type' => 'checkbox',
                'default' => 'no',
            );

            $this->form_fields[$replacedurl.'besteron_cid_cz'] =  array(
                'title' => __('CID (CZ)', 'besteron-payment-gateway'),
                'type' => 'text',
                'desc_tip' => __('Your Merchant CID from your BESTERON account Integration page', 'besteron-payment-gateway'),
                'default' => '',
            );

            $this->form_fields[$replacedurl.'besteron_key_cz'] = array(
                'title' => __('KEY (CZ)', 'besteron-payment-gateway'),
                'type' => 'password',
                'desc_tip' => __('Your Merchant Key from your BESTERON account Integration page', 'besteron-payment-gateway'),
            );

            $this->form_fields[$replacedurl.'besteron_api_cz'] = array(
                'title' => __('API (CZ)', 'besteron-payment-gateway'),
                'type' => 'password',
                'desc_tip' => __('Enter the Besteron API key.', 'besteron-payment-gateway'),
            );


        if(function_exists('get_sites')){

            $ar = [];



            $blogs = get_sites();
            foreach( $blogs as $b ){
                $ar[] = $b->domain;
            }  
            
            $array = [];

            foreach ($ar as $a ) {
                $cid =  'besteron_cid_'.strtolower($a);
                $key =  'besteron_key_'.strtolower($a);
                $api =  'besteron_api_'.strtolower($a);

                $array[$cid] = [
                    'title' => 'CID (' . $a . ')', 
                    'type' => 'text',
                    'desc_tip' => __('Your Merchant CID from your BESTERON account Integration page', 'besteron-payment-gateway'),
                    'default' => '',

                ];

                array_push($this->form_fields, $array[$cid]);
                $array[$key] = [
                    'title' => 'KEY (' . $a . ')', 
                    'type' => 'text',
                    'desc_tip' => __('Your Merchant CID from your BESTERON account Integration page', 'besteron-payment-gateway'),
                    'default' => '',

                ];
                array_push($this->form_fields, $array[$key]);

                $array[$api] = [
                    'title' => 'API (' . $a . ')', 
                    'type' => 'text',
                    'desc_tip' => __('Enter the Besteron API key.', 'besteron-payment-gateway'),
                    'default' => '',

                ];
                array_push($this->form_fields, $array[$api]);
            }
        }
    }


    // Submit payment and handle response
    public function process_payment($order_id) {
        global $woocommerce;

        if ( $this->debug_mode ) debug_to_console('function process_payment start'); //debug 

        include_once dirname(__FILE__) . '/lib/BesteronClass.php';

        // Get this Order's information so that we know
        // who to charge and how much
        $order = new WC_Order($order_id);


        $items = $order->get_items();

        //ver 1.1.1 - nastavenie ID objednavky alebo Cislo objednavky - ak nieco meni cislo objednavky
        if ( $this->vs_type == "yes" ) {
            //ak je povolene cislo objednavky v settings besteron
            $vs = $order->get_order_number();
            if (!is_numeric($vs) || (strlen($vs) > 10)) {
                // ak variabilny symbol nie je cislo alebo je dlhsi ako 10znakov, tak to bude ID objednavky
                $vs = str_pad($order_id, 10, "0", STR_PAD_LEFT);
            }
        } else {
            $vs = str_pad($order_id, 10, "0", STR_PAD_LEFT);
        }

        if ( $this->debug_mode ) debug_to_console('variable symbol - '.$vs); //debug 

        $amnt = $order->get_total();

        //return on Order Received page
        $ru = $this->get_return_url($order);

        // Multicurency support
        // WC curecny
        //$wc_currency = $order->get_currency(); // alternative
        $wc_currency = get_woocommerce_currency(); 
        if ( $this->debug_mode ) debug_to_console('wc_currency (get_woocommerce_currency) - '.$wc_currency); //debug 
        $order_currency = array_search($wc_currency, Besteron::getCurrList()); 

        //$bic = $_POST['besteron_bic'];
        $bic = isset($_POST['besteron_bic']) ? $_POST['besteron_bic'] : "";

        // get currency number from list
        $currency = array_search($wc_currency, Besteron::getCurrList()); 

        // get payment type - inline / redirect
        $payment_type = $this->payment_type;
        // get inline payment type - only card or card&buttons

        $replacedurl = site_url();
        $replacedurl = str_replace('https://', '', $replacedurl);
        $replacedurl = str_replace('.', '', $replacedurl);
        $key = $replacedurl . '_inline_payment_type';

        $inline_payment_type = $this->$key; // CARD or CARDANDBUTTON

        // testing if multi currency is enabled 
        if ( $this->multi_curr_var ) {
            // get select (possible) currencies from settings
            $curr_possible = $this->allowed_curr;
            // if not select items, get all
            if ( empty($curr_possible) ) {
                // all possible
                $curr_possible = Besteron::getCurrPossible(); 
            }
        } else {
            // default is all possible
            $curr_possible = Besteron::getCurrPossible(); 
        }

        if ( $this->debug_mode ) debug_to_console('payment_type - '.$payment_type); //debug 

        if ( $payment_type == 'redirect' ) {


            if($bic == '' && $this->$key == 'CARD' && $this->enabled_card_var){
                $bic = 'SPOROCARD';
            }

            // multicurrency support only for credit card
            if ($bic == 'SPOROCARD' || $bic == 'APPLEPAY' || $bic == 'GOOGLEPAY' || !$this->$enabled_card_var) {
                // credit card
            } else {
                // redirect - only selected banks

                $replacedurl = site_url();
                $replacedurl = str_replace('https://', '', $replacedurl);
                $replacedurl = str_replace('.', '', $replacedurl);
                $key_banks = $replacedurl . '_allowed_banks';

                $banks_possible = Besteron::getAllowBanks($this->$key_banks, $this->enabled_card_var, $this->support_banks, $this->enabled_bank_buttons_var, '',$this->apple_pay);
                $bank_curr = $banks_possible[$bic]['curr'];
                // get currency bank
                if ($bank_curr == $currency) {
                    $currency = $bank_curr; 
                } else {
                    $this->validBic($bic);
                    $msg = sprintf(__('Selected payment gateway does not support "%s" currency.', 'besteron-payment-gateway'), $wc_currency);
                    $msg = json_encode($banks_possible);
                    
                    return;
                }
            }

            if ( !in_array($currency, $curr_possible) ) {
                $msg = sprintf(__('Selected payment gateway does not support "%s" currency.', 'besteron-payment-gateway'), $wc_currency);
                wc_add_notice($msg, 'error');
                return;
            }
        } else {
            // inline payment

            if ( empty($inline_payment_type) ) {
                // if not, set to default
                $inline_payment_type = Besteron::PAYMENT_TYPE_DEFAULT; 
            }

            if ( $inline_payment_type == Besteron::PAYMENT_TYPE_DEFAULT ) {
                // CARD - support all currency
                $currency = $currency;
            } else {
                // CARDANDBUTTONS
                // auto set currency by country
                $banks_possible = Besteron::getSupportBanksPossible();
                $country = $this->inline_payment_cid_key;
                if ( $country == 'auto') {
                    foreach ($banks_possible as $key => $sup_bank) {
                        if ( $sup_bank['curr'] == $currency ) {
                            $currency = $sup_bank['curr'];
                        }
                    }
                } else {
                    // get currecny from inline_payment_cid_key
                    $currency = $banks_possible[$country]['curr'];
                }
            }

            // if currency is not in possible list, show error and return
            if ( !in_array($currency, $curr_possible) ) {
                $msg = sprintf(__('Selected payment gateway does not support "%s" currency.', 'besteron-payment-gateway'), $currency);
                wc_add_notice($msg, 'error');
                return;
            };
        }       

        // language for Besteron gateway
        $lang_best = $this->getSiteLanguage();

        $cid = $this->getBankCIDKEY('cid', $bic, $order_currency); // get CID
        $key = $this->getBankCIDKEY('key', $bic, $order_currency); // get KEY

        try {
            if ( $payment_type == 'redirect' ) {
                $this->validBic($bic);
            }


            if ( $this->vs_type == "yes" ) {

                $besteron = new BesteronRequest(
                    array(
                        Besteron::PARAM_CID     => $cid,
                        Besteron::PARAM_KEY     => $key,
                        Besteron::PARAM_DEMO_MODE => $this->besteron_demo_var,
                        Besteron::PARAM_AMNT    => floatval($amnt),
                        Besteron::PARAM_CURR    => $currency,
                        Besteron::PARAM_VS      => substr($order->get_order_number(),0,100),
                        Besteron::PARAM_RU      => $ru,
                        Besteron::PARAM_EMAIL   => $order->get_billing_email(),
                        Besteron::PARAM_LANG    => $lang_best,
                        Besteron::PARAM_PAYMENT_TYPE => $payment_type,
                        Besteron::PARAM_INLINE_PAYMENT_TYPE => $inline_payment_type,
                        Besteron::VALIDITY_TIME => $this->validity_time,
                        Besteron::ADDITIONAL_PARAM_NAME => $this->additional_param_name,
                        Besteron::ADDITIONAL_PARAM_VALUE => $this->additional_param_value
                    )
                );
            }
            else{
                $besteron = new BesteronRequest(
                    array(
                        Besteron::PARAM_CID     => $cid,
                        Besteron::PARAM_KEY     => $key,
                        Besteron::PARAM_DEMO_MODE => $this->besteron_demo_var,
                        Besteron::PARAM_AMNT    => floatval($amnt),
                        Besteron::PARAM_CURR    => $currency,
                        Besteron::PARAM_VS      => strval($order->id),
                        Besteron::PARAM_RU      => $ru,
                        Besteron::PARAM_EMAIL   => $order->get_billing_email(),
                        Besteron::PARAM_LANG    => $lang_best,
                        Besteron::PARAM_PAYMENT_TYPE => $payment_type,
                        Besteron::PARAM_INLINE_PAYMENT_TYPE => $inline_payment_type,
                        Besteron::VALIDITY_TIME => $this->validity_time,
                        Besteron::ADDITIONAL_PARAM_NAME => $this->additional_param_name,
                        Besteron::ADDITIONAL_PARAM_VALUE => $this->additional_param_value
                    )
                );
            }
            if ( $payment_type == 'redirect' ) {

               

                $replacedurl = site_url();
                $replacedurl = str_replace('https://', '', $replacedurl);
                $replacedurl = str_replace('.', '', $replacedurl);
                $key_banks = $replacedurl . '_allowed_banks';
                // redirect payment
                $url = $besteron->getUrl( $bic, Besteron::getAllowBanks($this->$key_banks, $this->enabled_card_var, $this->support_banks, $this->enabled_bank_buttons_var, '',$this->apple_pay),$order,$woocommerce->cart->get_cart(), Besteron::getArrayOfBanks($this->$key_banks, $this->enabled_card_var, $this->support_banks, $this->enabled_bank_buttons_var, '',$this->apple_pay,$inline_payment_type),$inline_payment_type);


               //  debug_to_console($url);

            } else {
                // inline payment
                $paymentData = [
                    'ru' => $ru,
                    'cid' => trim($cid),
                    'amt' => str_replace(',', '.', sprintf('%.2f', $amnt)),
                    'method' => $inline_payment_type,
                    'curr' => $currency,
                    'vs' => $vs,
                    'email' => $order->get_billing_email(),
                    'sign' => $besteron->generateSign(trim($cid).$inline_payment_type.str_replace(',', '.', sprintf('%.2f', $amnt)).$currency.$vs.$ru),
                    'lang' => $lang_best,
                    'virtual' => true /*$this->besteron_demo_var*/
                ];
            }

        } catch (Exception $e) {
           
            wc_add_notice($e->getMessage(), 'error');
            return false;
        }

        if ( $payment_type == 'redirect' ) {
            return array(
                'result' => 'success',
                'redirect' => $url
            );
        } else {
            return array(
                'result' => 'success',
                'reload' => false,
                'redirect' => false,
                'paymentData' => $paymentData
            );
        }
    }


    // Validate fields
    public function validate_fields() {

        return true;
    }


    // Check if we are forcing SSL on checkout pages
    // Custom function not required by the Gateway
    public function do_ssl_check() {
        if ($this->demo == "yes") {
            if (get_option('woocommerce_force_ssl_checkout') == "no") {
                echo "<div class='error'><p>" . sprintf(__("<strong>%s</strong> is enabled and WooCommerce is not forcing the SSL certificate on your checkout page. Please ensure that you have a valid SSL certificate and that you are <a href='%s'>forcing the checkout pages to be secured.</a>", 'besteron-payment-gateway'), $this->method_title, admin_url('admin.php?page=wc-settings&tab=checkout')) . "</p></div>";
            }
        }
    }

    public function payment_fields() {
        
         $replacedurl = site_url();
        $replacedurl = str_replace('https://', '', $replacedurl);
        $replacedurl = str_replace('.', '', $replacedurl);
        $key = $replacedurl . '_inline_payment_type';

        // debug
        if ( $this->debug_mode ) debug_to_console('function payment_fields start'); 

        // get payment type - inline / redirect
        $payment_type = $this->payment_type;
        if ( $payment_type == 'inline' ) {

            // echo '<div id="besteron-iframe"></div>'; // important for add ifreme with js !
            $description = $this->get_description();
            // $type_info = Besteron::getPaymentTypeList(); // show with payment type
            if ( $description ) {
                //echo wpautop( wptexturize( $description .' '. $type_info[$payment_type].'.' ) ); // show with payment type
                echo wpautop( wptexturize( $description ) ); 
            }

        } else {

            $img_url =  plugins_url('assets/images/', __FILE__);
            $img_src = '0.png';

            $wc_currency = get_woocommerce_currency(); 
            $currency = array_search($wc_currency, Besteron::getCurrList()); 

            $replacedurl = site_url();
            $replacedurl = str_replace('https://', '', $replacedurl);
            $replacedurl = str_replace('.', '', $replacedurl);
            $key_banks = $replacedurl . '_allowed_banks';

            // get possible banks with card
            if ( $this->enabled_bank_buttons_var ) {
                // with allowed_banks
                $banks_possible = Besteron::getAllowedBanks($this->$key_banks, $this->enabled_card_var, $this->support_banks, true, $currency,$this->apple_pay,$this->$key);
            } else {
                // without allowed_banks
                $banks_possible = Besteron::getAllowedBanks($this->$key_banks, $this->enabled_card_var, $this->support_banks, false, $currency,$this->apple_pay,$this->$key);
            }

            echo '<div class="form-row form-row-wide" style="clear: both; float:left;">';

            foreach ($banks_possible as $bic => $bank) {
                $radio_id = 'besteron_bic_' . $bic;
                //TODO: set checked
                $checked = false ? 'checked="checked"' : '';
                if ( count($banks_possible) < 2 ) {
                    // if only one, check one
                    $checked = 'checked="checked"'; 
                }
                $radio_name = "besteron_bic";

                //$bank['label'] = $bank['label'] .' <span style="font-size: 0.8em;">('. $bank['support'].')</span>'; // add language sign
                if ($bic == 'SPOROCARD') $bank['label'] = __( 'Card payment', 'besteron-payment-gateway' ); // sanitize static array translate
                echo "<div style='display:table; vertical-align: middle; text-align:center;'>"
                    . "<div style='display:table-cell; vertical-align: middle; padding-right: 10px;'>"
                    . "<input class='besteron-bic' type='radio' id='$radio_id' name='$radio_name' value='{$bic}' $checked />"
                    . "</div>"
                    . "<span style='display:table-cell; vertical-align: middle; text-align: left; width: 146px; line-height: 1.2em;'>"
                    . "{$bank['label']}"
                    . "</span>"
                    . "<label for='$radio_id'>"
                    . "<div style='margin-left:10px; float:left; display:table-cell; vertical-align:middle;'>"
                    . "<img src='{$img_url}/{$bank['image']}/{$img_src}' alt='{$bank['name']}' style='margin: auto; vertical-align: middle; max-height:65px;'/>"
                    . "</div>"
                    . "</label>"
                    . "</div>";
            }
            if(sizeof($banks_possible)==0){
                echo "Platba kartou alebo online bankovým prevodom cez platobnú bránu Besteron";
            }
            echo '</div>';
            echo '<div style="clear: both"></div>'; 
        }
    }
    // validate BIC/SWIFT Code, url param TYPE
    // @return boolean
    // @throws Exception
    private function validBic($bic) {
        if(!$this->enabled_card_var){
            return true;
        }
        if (!$bic) {
            $msg = __('Besteron: Bank is not selected. Select please your bank.', 'besteron-payment-gateway');
            throw new Exception($msg);
        }
        $replacedurl = site_url();
        $replacedurl = str_replace('https://', '', $replacedurl);
        $replacedurl = str_replace('.', '', $replacedurl);
        $key_banks = $replacedurl . '_allowed_banks';

        if (!array_key_exists( $bic, Besteron::getAllowBanks($this->$key_banks, $this->enabled_card_var, $this->support_banks, $this->enabled_bank_buttons_var, '',$this->apple_pay) )) {
            $msg = __('Besteron: Unknown BIC/SWIFT', 'besteron-payment-gateway') . ': ' . $bic;
            ;
            throw new Exception($msg);
        }
        return true;
    }

}
